unit main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls, StdCtrls,
  Spin, Math,
  mystrings, PgsClass;

type

  { TMainForm }

  TMainForm = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Edit_iTol: TEdit;
    Edit_zMinSun: TEdit;
    Edit_zMinPlanet: TEdit;
    Edit_ireq: TEdit;
    Edit_z1min: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    MainMemo: TMemo;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FloatInput(Sender: TObject; var Key: char);
    procedure FormCreate(Sender: TObject);
    procedure IntInput(Sender: TObject; var Key: char);
  private
    MyPgs: tPgs;
    procedure CheckCylGearsets;
    procedure CheckPgs;
    procedure CalcPgsZS(azZmin, azPmin: integer; i_req, adeltai: extended; var aPgs: tPgs);
    procedure CalcPgsZH(azZmin, azPmin: integer; i_req, adeltai: extended; var aPgs: tPgs);
    procedure CalcPgsHS(azZmin, azPmin: integer; i_req, adeltai: extended; var aPgs: tPgs); 
    procedure CalcPgsSZ(azZmin, azPmin: integer; i_req, adeltai: extended; var aPgs: tPgs); 
    procedure CalcPgsHZ(azZmin, azPmin: integer; i_req, adeltai: extended; var aPgs: tPgs);
    procedure CalcPgsSH(azZmin, azPmin: integer; i_req, adeltai: extended; var aPgs: tPgs);
  public

  end;

var
  MainForm: TMainForm;

implementation

const
  i_minZS =  2.19;
  i_minZH = -1.19;
  i_minHS =  1.04;
  i_maxHS =  1.83;
  i_minSZ =  0.04;
  i_maxSZ =  0.45;
  i_minHZ = -0.83;
  i_maxHZ = -0.04;
  i_minSH =  0.55;
  i_maxSH =  0.96;

{$R *.lfm}

{ TMainForm }

procedure TMainForm.FloatInput(Sender: TObject; var Key: char);
begin
  MyFloatInput(Sender, Key);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  MyPgs := tPgs.Create(Self);
end;

procedure TMainForm.IntInput(Sender: TObject; var Key: char);
begin
  MyIntInput(Sender, Key);
end;

procedure TMainForm.Button1Click(Sender: TObject);
begin
  CheckCylGearsets;
  CheckPgs;
end;

procedure TMainForm.Button2Click(Sender: TObject);
begin
  MainMemo.Clear;
end;

procedure TMainForm.CheckCylGearsets;
var
  delta_i: extended;
  i_real: extended;
  i_req: extended;
  z_1, z_2: integer;
  z_1min: integer;
begin
  delta_i := Str2Float(Edit_iTol.Text);
  i_req := Str2Float(Edit_ireq.Text);
  z_1min := StrToInt(Edit_z1min.Text);
  z_1 := z_1min;
  z_2 := round(i_req * z_1);
  i_real := z_2 / z_1;
  while (abs(i_req - i_real) > delta_i) do
  begin
    z_1 := z_1 + 1;
    z_2 := round(i_req * z_1);
    i_real := z_2 / z_1;
  end;

  MainMemo.Lines.Add('');
  MainMemo.Lines.Add('- - - Cylindrical gearset - - -');
  MainMemo.Lines.Add(Int2Str(z_1, 10) + Int2Str(z_2, 10) + Float2Str(i_real, 16, 6));

end;


procedure TMainForm.CheckPgs;
var
  delta_i: extended;
  i_req: extended;
  z_Zmin: integer;
  z_Pmin: integer;
begin
  i_req := Str2Float(Edit_ireq.Text);
  if i_req < 0 then i_req := i_req * -1;
  delta_i := Str2Float(Edit_iTol.Text);
  z_Zmin := StrToInt(Edit_zMinSun.Text);
  z_Pmin := StrToInt(Edit_zMinPlanet.Text);
  MainMemo.Lines.Add('');
  MainMemo.Lines.Add('- - - Planetary gearsets: - - -');
  { i_ZS:  input sun, output planet carrier }
  if i_req >= i_minZS then
  begin
    CalcPgsZS(z_Zmin, z_Pmin, i_req, delta_i, MyPgs);
    with MyPgs do
    begin
      MainMemo.Lines.Add('');
      MainMemo.Lines.Add('      sun - planet carrier: ' + Int2Str(Sun.z, 4) + Int2Str(Planet.z, 8) + Int2Str(Ring.z, 8) + Float2Str(i_ZS, 13, 6));
    end;
  end else
  begin
    MainMemo.Lines.Add('');
    MainMemo.Lines.Add('      sun - planet carrier: no solution!');
  end;

  { i_ZH: input sun, output ring gear }
  if -i_req <= i_minZH then
  begin
    CalcPgsZH(z_Zmin, z_Pmin, -i_req, delta_i, MyPgs);
    with MyPgs do
    begin
      MainMemo.Lines.Add('');
      MainMemo.Lines.Add('           sun - ring gear: ' + Int2Str(Sun.z, 4) + Int2Str(Planet.z, 8) + Int2Str(Ring.z, 8) + Float2Str(i_ZH, 13, 6));
    end;
  end else
  begin
    MainMemo.Lines.Add('');
    MainMemo.Lines.Add('           sun - ring gear: no solution!');
  end;

  { i_HS: ring gear - planet carrier }
  if (i_req >= i_minHS) and (i_req <= i_maxHS) then
  begin
    CalcPgsHS(z_Zmin, z_Pmin, i_req, delta_i, MyPgs);
    with MyPgs do
    begin
      MainMemo.Lines.Add('');
      MainMemo.Lines.Add('ring gear - planet carrier: ' + Int2Str(Sun.z, 4) + Int2Str(Planet.z, 8) + Int2Str(Ring.z, 8) + Float2Str(i_HS, 13, 6));
    end;
  end else
  begin
    MainMemo.Lines.Add('');
    MainMemo.Lines.Add('ring gear - planet carrier: no solution!');
  end;

  { i_SZ: planet carrier - sun gear }
  if (i_req >= i_minSZ) and (i_req <= i_maxSZ) then
  begin
    CalcPgsSZ(z_Zmin, z_Pmin, i_req, delta_i, MyPgs);
    with MyPgs do
    begin
      MainMemo.Lines.Add('');
      MainMemo.Lines.Add('planet carrier - sun: ' + Int2Str(Sun.z, 4) + Int2Str(Planet.z, 8) + Int2Str(Ring.z, 8) + Float2Str(i_SZ, 13, 6));
    end;
  end else
  begin
    MainMemo.Lines.Add('');
    MainMemo.Lines.Add('      planet carrier - sun: no solution!');
  end; 

  { i_HZ: input ring gear, output sun }
  if (-i_req >= i_minHZ) and (-i_req <= i_maxHZ) then
  begin
    CalcPgsHZ(z_Zmin, z_Pmin, -i_req, delta_i, MyPgs);
    with MyPgs do
    begin
      MainMemo.Lines.Add('');
      MainMemo.Lines.Add('           ring gear - sun: ' + Int2Str(Sun.z, 4) + Int2Str(Planet.z, 8) + Int2Str(Ring.z, 8) + Float2Str(i_HZ, 13, 6));
    end;
  end else
  begin
    MainMemo.Lines.Add('');
    MainMemo.Lines.Add('           ring gear - sun: no solution!');
  end;  

  { i_SH: planet carrier - sun gear }
  if (i_req >= i_minSH) and (i_req <= i_maxSH) then
  begin
    CalcPgsSH(z_Zmin, z_Pmin, i_req, delta_i, MyPgs);
    with MyPgs do
    begin
      MainMemo.Lines.Add('');
      MainMemo.Lines.Add('planet carrier - ring gear: ' + Int2Str(Sun.z, 4) + Int2Str(Planet.z, 8) + Int2Str(Ring.z, 8) + Float2Str(i_SH, 13, 6));
    end;
  end else
  begin
    MainMemo.Lines.Add('');
    MainMemo.Lines.Add('planet carrier - ring gear: no solution!');
  end;
end;

procedure TMainForm.CalcPgsZS(azZmin, azPmin: integer; i_req, adeltai: extended; var aPgs: tPgs);
var
  z_Z, z_P, z_H: integer;
  iZS, iZS1, iZS2, iZS3: extended;
begin
  with aPgs do
  begin
    if i_req < 0 then i_req := -1 * i_req;
    z_Z := azZmin;
    z_H := round((1 - i_req) * z_Z);
    z_P := round((-z_Z - z_H) / 2);
    iZS1 := 1 - (z_H + 1) / z_Z;
    iZS2 := 1 - z_H / z_Z;
    iZS3 := 1 - (z_H - 1) / z_Z;
    iZS := iZS1;
    if abs(i_req - iZS2) < abs(i_req - iZS) then iZS := iZS2;
    if abs(i_req - iZS3) < abs(i_req - iZS) then iZS := iZS3;
    while (abs(i_req - iZS) > adeltai) or (z_P < azPmin) do
    begin
      z_Z := z_Z + 1;
      z_H := round((1 - i_req) * z_Z);
      iZS1 := 1 - (z_H + 1) / z_Z;
      iZS2 := 1 - z_H / z_Z;
      iZS3 := 1 - (z_H - 1) / z_Z;
      iZS := iZS1;
      if abs(i_req - iZS2) <= abs(i_req - iZS) then iZS := iZS2;
      if abs(i_req - iZS3) <= abs(i_req - iZS) then iZS := iZS3;
      z_P := ceil((-z_Z - z_H) / 2);
    end;
  end;
  aPgs.Sun.z := z_Z;
  aPgs.Planet.z := z_P;
  aPgs.Ring.z := z_H;
  aPgs.i_ZS := iZS;
end;


procedure TMainForm.CalcPgsZH(azZmin, azPmin: integer; i_req, adeltai: extended; var aPgs: tPgs);
var
  z_Z, z_P, z_H: integer;
  iZH, iZH1, iZH2, iZH3: extended;
begin
  with aPgs do
  begin
    if i_req > 0 then i_req := -1 * i_req;
    z_Z := azZmin;
    z_H := round(z_Z * i_req);
    z_P := ceil((-z_Z - z_H) / 2);
    iZH1 := (z_H + 1) / z_Z;
    iZH2 := z_H / z_Z;
    iZH3 := (z_H - 1) / z_Z;
    iZH := iZH1;
    if abs(i_req - iZH2) < abs(i_req - iZH) then iZH := iZH2;
    if abs(i_req - iZH3) < abs(i_req - iZH) then iZH := iZH3;
    while (abs(i_req - iZH) > adeltai) or (z_P < azPmin) do
    begin
      z_Z := z_Z + 1;
      z_H := round(z_Z * i_req);
      iZH1 := (z_H + 1) / z_Z;
      iZH2 := z_H / z_Z;
      iZH3 := (z_H - 1) / z_Z;
      iZH := iZH1;
      if abs(i_req - iZH2) < abs(i_req - iZH) then iZH := iZH2;
      if abs(i_req - iZH3) < abs(i_req - iZH) then iZH := iZH3;
      z_P := ceil((-z_Z - z_H) / 2);
    end;
  end;
  aPgs.Sun.z := z_Z;
  aPgs.Planet.z := z_P;
  aPgs.Ring.z := z_H;
  aPgs.i_ZH := iZH;
end;   


procedure TMainForm.CalcPgsHS(azZmin, azPmin: integer; i_req, adeltai: extended; var aPgs: tPgs);
var
  z_Z, z_P, z_H: integer;
  iHS, iHS1, iHS2, iHS3: extended;
begin
  with aPgs do
  begin
    if i_req < 0 then i_req := -1 * i_req;
    z_Z := azZmin;
    z_H := round(z_Z / (1 - i_req));
    z_P := ceil((-z_Z - z_H) / 2);
    iHS1 := (z_H + 1 - z_Z) / z_H;
    iHS2 := (z_H - z_Z) / z_H;
    iHS3 := (z_H - 1 - z_Z) / z_H;
    iHS := iHS1;
    if abs(i_req - iHS2) < abs(i_req - iHS) then iHS := iHS2;
    if abs(i_req - iHS3) < abs(i_req - iHS) then iHS := iHS3;
    while (abs(i_req - iHS) > adeltai) or (z_P < azPmin) do
    begin
      z_Z := z_Z + 1;
      z_H := round(z_Z / (1 - i_req));
      iHS1 := (z_H + 1 - z_Z) / z_H;
      iHS2 := (z_H - z_Z)/ z_H;
      iHS3 := (z_H - 1 - z_Z) / z_H;
      iHS := iHS1;
      if abs(i_req - iHS2) < abs(i_req - iHS) then iHS := iHS2;
      if abs(i_req - iHS3) < abs(i_req - iHS) then iHS := iHS3;
      z_P := ceil((-z_Z - z_H) / 2);
    end;
  end;
  aPgs.Sun.z := z_Z;
  aPgs.Planet.z := z_P;
  aPgs.Ring.z := z_H;
  aPgs.i_HS := iHS;
end;


procedure TMainForm.CalcPgsSZ(azZmin, azPmin: integer; i_req, adeltai: extended; var aPgs: tPgs);
var
  z_Z, z_P, z_H: integer;
  iSZ, iSZ1, iSZ2, iSZ3: extended;
begin
  with aPgs do
  begin
    if i_req < 0 then i_req := -1 * i_req;
    z_Z := azZmin;
    z_H := round((1 - 1 / i_req) * z_Z);
    z_P := ceil((-z_Z - z_H) / 2);
    iSZ1 := 1 / (1 - (z_H + 1) / z_Z);
    iSZ2 := 1 / (1 - z_H / z_Z);
    iSZ3 := 1 / (1 - (z_H - 1) / z_Z);
    iSZ := iSZ1;
    if abs(i_req - iSZ2) < abs(i_req - iSZ) then iSZ := iSZ2;
    if abs(i_req - iSZ3) < abs(i_req - iSZ) then iSZ := iSZ3;
    while (abs(i_req - iSZ) > adeltai) or (z_P < azPmin) do
    begin
      z_Z := z_Z + 1;
      z_H := round((1 - 1 / i_req) * z_Z);
      iSZ1 := 1 / (1 - (z_H + 1) / z_Z);
      iSZ2 := 1 / (1 - (z_H) / z_Z);
      iSZ3 := 1 / (1 - (z_H - 1) / z_Z);
      iSZ := iSZ1;
      if abs(i_req - iSZ2) < abs(i_req - iSZ) then iSZ := iSZ2;
      if abs(i_req - iSZ3) < abs(i_req - iSZ) then iSZ := iSZ3;
      z_P := ceil((-z_Z - z_H) / 2);
    end;
  end;
  aPgs.Sun.z := z_Z;
  aPgs.Planet.z := z_P;
  aPgs.Ring.z := z_H;
  aPgs.i_SZ := iSZ;
end;


procedure TMainForm.CalcPgsHZ(azZmin, azPmin: integer; i_req, adeltai: extended; var aPgs: tPgs);
var
  z_Z, z_P, z_H: integer;
  iHZ, iHZ1, iHZ2, iHZ3: extended;
begin
  with aPgs do
  begin
    if i_req > 0 then i_req := -1 * i_req;
    z_Z := azZmin;
    z_H := round(z_Z / i_req);
    z_P := ceil((-z_Z - z_H) / 2);
    iHZ1 := z_Z / (z_H + 1);
    iHZ2 := z_Z / z_H;
    iHZ3 := z_Z / (z_H - 1);
    iHZ := iHZ1;
    if abs(i_req - iHZ2) < abs(i_req - iHZ) then iHZ := iHZ2;
    if abs(i_req - iHZ3) < abs(i_req - iHZ) then iHZ := iHZ3;

    //MainMemo.Lines.Add('1st step: ' + Int2Str(z_Z, 4) + Int2Str(z_P, 8) + Int2Str(z_H, 8) + Float2Str(iHZ, 18, 10));

    while (abs(i_req - iHZ) > adeltai) or (z_P < azPmin) do
    begin
      z_Z := z_Z + 1;
      z_H := round(z_Z / i_req);
      iHZ1 := z_Z / (z_H + 1);
      iHZ2 := z_Z / z_H;
      iHZ3 := z_Z / (z_H - 1);
      iHZ := iHZ1;
      if abs(i_req - iHZ2) < abs(i_req - iHZ) then iHZ := iHZ2;
      if abs(i_req - iHZ3) < abs(i_req - iHZ) then iHZ := iHZ3;
      z_P := ceil((-z_Z - z_H) / 2);

      //MainMemo.Lines.Add('next step: ' + Int2Str(z_Z, 4) + Int2Str(z_P, 8) + Int2Str(z_H, 8) + Float2Str(iHZ, 18, 10));

    end;
  end;
  aPgs.Sun.z := z_Z;
  aPgs.Planet.z := z_P;
  aPgs.Ring.z := z_H;
  aPgs.i_HZ := iHZ;
end;


procedure TMainForm.CalcPgsSH(azZmin, azPmin: integer; i_req, adeltai: extended; var aPgs: tPgs);
var
  z_Z, z_P, z_H: integer;
  iSH, iSH1, iSH2, iSH3: extended;
begin
  with aPgs do
  begin
    if i_req < 0 then i_req := -1 * i_req;
    z_Z := azZmin;
    z_H := round((1 - 1 / i_req) * z_Z);
    z_P := ceil((-z_Z - z_H) / 2);
    iSH1 := 1 / (1 - (z_H + 1) / z_Z);
    iSH2 := 1 / (1 - z_H / z_Z);
    iSH3 := 1 / (1 - (z_H - 1) / z_Z);
    iSH := iSH1;
    if abs(i_req - iSH2) < abs(i_req - iSH) then iSH := iSH2;
    if abs(i_req - iSH3) < abs(i_req - iSH) then iSH := iSH3;
    while (abs(i_req - iSH) > adeltai) or (z_P < azPmin) do
    begin
      z_Z := z_Z + 1;
      z_H := round((1 - 1 / i_req) * z_Z);
      iSH1 := 1 / (1 - (z_H + 1) / z_Z);
      iSH2 := 1 / (1 - (z_H) / z_Z);
      iSH3 := 1 / (1 - (z_H - 1) / z_Z);
      iSH := iSH1;
      if abs(i_req - iSH2) < abs(i_req - iSH) then iSH := iSH2;
      if abs(i_req - iSH3) < abs(i_req - iSH) then iSH := iSH3;
      z_P := ceil((-z_Z - z_H) / 2);
    end;
  end;
  aPgs.Sun.z := z_Z;
  aPgs.Planet.z := z_P;
  aPgs.Ring.z := z_H;
  aPgs.i_SH := iSH;
end;

end.

